/*======================================================================================================================
 *      File: RemoteOutlet.cpp
 *   Project: Remote Outlet
 *    Author: Jared Julien <jaredjulien@exsystems.net>
 * Copyright: (c) 2014-2017 Jared Julien, eX Systems
 * --------------------------------------------------------------------------------------------------------------------
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
 * documentation files (the "Software"), to deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies or substantial portions of the
 * Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
 * OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 * --------------------------------------------------------------------------------------------------------------------
 * Date        Rev   Author    Change Description
 * ==========  ====  ========  ========================================================================================
 * 2014-10-15  1     Jared     Initial file creation
 * 2017-12-13  2     Jared     Added beep on special code for when the device is selected on the remote
 * 2017-12-13  3     Jared     Changed relay pin mapping to better suit order on outlets (reads like a book)
 * --------------------------------------------------------------------------------------------------------------------
 */
//======================================================================================================================
// Header Files
//----------------------------------------------------------------------------------------------------------------------
#include "Arduino.h"

#include <IRremote.h>
#include <TimerFreeTone.h>

#include "Relay.h"


//======================================================================================================================
// Definitions
//----------------------------------------------------------------------------------------------------------------------
#define PIN_SENSOR	2
#define PIN_SWITCH	3

#define PIN_RELAY_1	7
#define PIN_RELAY_2	4
#define PIN_RELAY_3	6
#define PIN_RELAY_4	5

#define PIN_STATUS	8
#define PIN_SPEAKER	9

#define RELAY_COUNT	4

#define BEEP_CODE 0xAF07639E


//======================================================================================================================
// Type Definitions
//----------------------------------------------------------------------------------------------------------------------
typedef struct {
	uint32_t code;
	Relay relay;
} output_t;


//======================================================================================================================
// Variables
//----------------------------------------------------------------------------------------------------------------------
IRrecv irrecv(PIN_SENSOR);
decode_results results;

output_t outputs[RELAY_COUNT] = {
	{0xA10C140B, Relay(PIN_RELAY_1)},
	{0xA10C940B, Relay(PIN_RELAY_2)},
	{0xA10C540B, Relay(PIN_RELAY_3)},
	{0xA10CD40B, Relay(PIN_RELAY_4)}
};


//======================================================================================================================
// Setup Function
//----------------------------------------------------------------------------------------------------------------------
void setup(void)
{
	uint8_t index;

	Serial.begin(19200);

	pinMode(PIN_STATUS, OUTPUT);
	digitalWrite(PIN_STATUS, HIGH);

	pinMode(PIN_SPEAKER, OUTPUT);

	for (index = 0; index < RELAY_COUNT; index++)
	{
		outputs[index].relay.begin();
	}

	irrecv.enableIRIn();

	Serial.println("Setup Complete");
}

//======================================================================================================================
// Loop Function
//----------------------------------------------------------------------------------------------------------------------
void loop(void)
{
	uint8_t index;

	// check if something has been received
	if (irrecv.decode(&results))
	{
		Serial.print("Code received: ");
		Serial.println(results.value, HEX);

		if (BEEP_CODE == results.value)
		{
			TimerFreeTone(PIN_SPEAKER, 3000, 50);
		}
		else
		{
			// loop through relay configurations
			for (index = 0; index < RELAY_COUNT; index++)
			{
				// check if a matching code has been found
				if (outputs[index].code == results.value)
				{
					Serial.print("Relay ");
					Serial.print(index + 1);
					if (outputs[index].relay.toggle())
					{
						Serial.println(" set to ON");
						TimerFreeTone(PIN_SPEAKER, 3000, 100);
					}
					else
					{
						Serial.println(" set to OFF");
						TimerFreeTone(PIN_SPEAKER, 2500, 100);
					}
				}
			}
		}

		irrecv.resume();
	}
}


/* End of file RemoteOutlet.cpp */
